@@ -103,10 +103,11 @@ INSTALLED_APPS = (
     'pages',
     'mptt',
     'staticfiles',
-    #'tinymce',
+    'tinymce',
+    'south',
 )
 
-#PAGE_TINYMCE = True
+PAGE_TINYMCE = True
 
 PAGE_CONNECTED_MODELS = [{
     'model':'documents.models.Document',
