@@ -51,6 +51,7 @@ def run_tests(test_labels=('pages',), verbosity=1, interactive=True,
         extra_tests)
     cov.stop()
     cov.html_report(modules, directory='coverage')
-
     sys.exit(results)
 
+if __name__ == '__main__':
+    run_tests()
\ No newline at end of file
