@@ -1,14 +1,11 @@
 # -*- coding: utf-8 -*-
 """Page Admin module."""
 from os.path import join
-from inspect import isclass, getmembers
 
-from django.forms import Widget, TextInput, Textarea, CharField
 from django.contrib import admin
 from django.utils.translation import ugettext as _, ugettext_lazy
 from django.utils.encoding import force_unicode
 from django.conf import settings as global_settings
-from django.db import models
 from django.http import HttpResponseRedirect
 from django.contrib.admin.util import unquote
 from django.contrib.admin.sites import AlreadyRegistered
@@ -19,11 +16,12 @@ from pages.http import get_language_from_request, get_template_from_request
 
 from pages.utils import get_placeholders
 from pages.utils import has_page_add_permission, get_language_from_request
+from pages.templatetags.pages_tags import PlaceholderNode
 from pages.admin.utils import get_connected, make_inline_admin
-from pages.admin import widgets
 from pages.admin.forms import PageForm
 from pages.admin.views import traduction, get_content, sub_menu
 from pages.admin.views import change_status, modify_content, delete_content
+import pages.admin.widgets
 
 class PageAdmin(admin.ModelAdmin):
     """Page Admin class."""
@@ -38,6 +36,7 @@ class PageAdmin(admin.ModelAdmin):
     #if getattr(settings, 'PAGE_USE_SITE_ID'):
     general_fields.append('sites')
     insert_point = general_fields.index('status') + 1
+    
     # Strange django behavior. If not provided, django will try to find
     # 'page' foreign key in all registered models
     inlines = []
@@ -51,8 +50,14 @@ class PageAdmin(admin.ModelAdmin):
     if settings.PAGE_SHOW_START_DATE:
         general_fields.insert(insert_point, 'publication_date')
 
+    from pages.urlconf_registry import registry
+    if(len(registry)):
+        general_fields.append('delegate_to')
+        insert_point = general_fields.index('status') + 1
+
     normal_fields = ['language']
-    if settings.PAGE_TEMPLATES:
+    page_templates = settings.get_page_templates()
+    if len(page_templates) > 0:
         normal_fields.append('template')
     normal_fields.append('redirect_to')
     normal_fields.append('redirect_to_url')
@@ -82,45 +87,6 @@ class PageAdmin(admin.ModelAdmin):
             'javascript/pages_form.js',
         )]
 
-    def __call__(self, request, url):
-        """
-        Delegate to the appropriate method, based on the URL.
-
-        DEPRECATED. This function is the old way of handling URL resolution, and
-        is deprecated in favor of real URL resolution -- see ``get_urls()``.
-        """
-
-        # Delegate to the appropriate method, based on the URL.
-        if url is None:
-            return self.list_pages(request)
-        elif url == 'jsi18n':
-            return self.i18n_javascript(request)
-        elif 'traduction' in url:
-            page_id, action, language_id = url.split('/')
-            return traduction(request, unquote(page_id), unquote(language_id))
-        elif 'get-content' in url:
-            page_id, action, content_id = url.split('/')
-            return get_content(request, unquote(page_id), unquote(content_id))
-        elif 'modify-content' in url:
-            page_id, action, content_id, language_id = url.split('/')
-            return modify_content(request, unquote(page_id),
-                                    unquote(content_id), unquote(language_id))
-        elif 'delete-content' in url:
-            page_id, action, language_id = url.split('/')
-            return delete_content(request, unquote(page_id), unquote(language_id))
-        elif url.endswith('/sub-menu'):
-            return sub_menu(request, unquote(url[:-9]))
-        elif url.endswith('/move-page'):
-            return self.move_page(request, unquote(url[:-10]))
-        elif url.endswith('/change-status'):
-            page_id, action = url.split('/')
-            return change_status(request, page_id)
-
-        ret = super(PageAdmin, self).__call__(request, url)
-
-        return ret
-
-
     def urls(self):
         from django.conf.urls.defaults import patterns, url, include
 
@@ -152,7 +118,7 @@ class PageAdmin(admin.ModelAdmin):
         """Displays the i18n JavaScript that the Django admin
         requires.
 
-        This takes into account the USE_I18N setting. If it's set to False, the
+        This takes into account the ``USE_I18N`` setting. If it's set to False, the
         generated JavaScript will be leaner and faster.
         """
         if global_settings.USE_I18N:
@@ -161,16 +127,17 @@ class PageAdmin(admin.ModelAdmin):
             from django.views.i18n import null_javascript_catalog as javascript_catalog
         return javascript_catalog(request, packages='pages')
 
-    def save_model(self, request, obj, form, change):
+    def save_model(self, request, page, form, change):
         """Move the page in the tree if necessary and save every
-        placeholder Content object.
+        placeholder :class:`Content <pages.models.Content>`.
         """
 
         language = form.cleaned_data['language']
         target = form.data.get('target', None)
         position = form.data.get('position', None)
-        obj.save()
-        
+        page.save()
+
+        # if True, we need to move the page
         if target and position:
             try:
                 target = self.model.objects.get(pk=target)
@@ -178,32 +145,24 @@ class PageAdmin(admin.ModelAdmin):
                 pass
             else:
                 target.invalidate()
-                obj.move_to(target, position)
+                page.move_to(target, position)
 
-        for mandatory_placeholder in self.mandatory_placeholders:
-            Content.objects.set_or_create_content(obj, language,
-                mandatory_placeholder, form.cleaned_data[mandatory_placeholder])
+        for name in self.mandatory_placeholders:
+            data = form.cleaned_data[name]
+            placeholder = PlaceholderNode(name)
+            placeholder.save(page, language, data, change)
 
-        for placeholder in get_placeholders(obj.get_template()):
-            if placeholder.name in form.cleaned_data:
-                if change:
-                    if placeholder.name not in self.mandatory_placeholders:
-                        # we need create a new content if revision is enabled
-                        if settings.PAGE_CONTENT_REVISION and placeholder.name \
-                                not in settings.PAGE_CONTENT_REVISION_EXCLUDE_LIST:
-                            Content.objects.create_content_if_changed(obj, language,
-                                placeholder.name, form.cleaned_data[placeholder.name])
-                        else:
-                            Content.objects.set_or_create_content(obj, language,
-                                placeholder.name, form.cleaned_data[placeholder.name])
-                else:
-                    Content.objects.set_or_create_content(obj, language,
-                        placeholder.name, form.cleaned_data[placeholder.name])
-
-        obj.invalidate()
+        for placeholder in get_placeholders(page.get_template()):
+            if(placeholder.name in form.cleaned_data and placeholder.name
+                    not in self.mandatory_placeholders):
+                data = form.cleaned_data[placeholder.name]
+                placeholder.save(page, language, data, change)
+        
+        page.invalidate()
 
     def get_fieldsets(self, request, obj=None):
-        """Add fieldsets of placeholders to the list of already
+        """
+        Add fieldsets of placeholders to the list of already
         existing fieldsets.
         """
         additional_fieldsets = []
@@ -234,22 +193,9 @@ class PageAdmin(admin.ModelAdmin):
             instance.author = request.user
         return instance
 
-    def get_widget(self, name, fallback=Textarea):
-        """Given the name of a placeholder return a Widget subclass
-        like Textarea or TextInput."""
-        if name and '.' in name:
-            name = str(name)
-            module_name, class_name = name.rsplit('.', 1)
-            module = __import__(module_name, {}, {}, [class_name])
-            widget = getattr(module, class_name, fallback)
-        else:
-            widget = dict(getmembers(widgets, isclass)).get(name, fallback)
-        if not isinstance(widget(), Widget):
-            widget = fallback
-        return widget
-
     def get_form(self, request, obj=None, **kwargs):
-        """Get PageForm for the Page model and modify its fields depending on
+        """Get a :class:`Page <pages.admin.forms.PageForm>` for the
+        :class:`Page <pages.models.Page>` and modify its fields depending on
         the request."""
         form = super(PageAdmin, self).get_form(request, obj, **kwargs)
 
@@ -263,39 +209,27 @@ class PageAdmin(admin.ModelAdmin):
             form.base_fields['slug'].label = _('Slug')
 
         template = get_template_from_request(request, obj)
-        if settings.PAGE_TEMPLATES:
-            template_choices = list(settings.PAGE_TEMPLATES)
+        page_templates = settings.get_page_templates()
+        if len(page_templates) > 0:
+            template_choices = list(page_templates)
             template_choices.insert(0, (settings.DEFAULT_PAGE_TEMPLATE,
                     _('Default template')))
             form.base_fields['template'].choices = template_choices
             form.base_fields['template'].initial = force_unicode(template)
 
         for placeholder in get_placeholders(template):
-            widget_class = self.get_widget(placeholder.widget)
-            try:
-                widget = widget_class(language=language)
-            except TypeError:
-                widget = widget_class()
-            if placeholder.parsed:
-                help_text = _('Note: This field is evaluated as template code.')
-            else:
-                help_text = ""
             name = placeholder.name
             if obj:
                 initial = Content.objects.get_content(obj, language, name)
             else:
                 initial = None
-            if name not in self.mandatory_placeholders:
-                form.base_fields[placeholder.name] = CharField(widget=widget,
-                    initial=initial, help_text=help_text, required=False)
-            else:
-                form.base_fields[name].initial = initial
-                form.base_fields[name].help_text = help_text
+            form.base_fields[name] = placeholder.get_field(obj, language, initial=initial)
 
         return form
 
     def change_view(self, request, object_id, extra_context=None):
-        """The 'change' admin view for the Page model."""
+        """The ``change`` admin view for the
+        :class:`Page <pages.models.Page>`."""
         language = get_language_from_request(request)
         extra_context = {
             'language': language,
@@ -321,7 +255,7 @@ class PageAdmin(admin.ModelAdmin):
                                                         extra_context)
 
     def add_view(self, request, form_url='', extra_context=None):
-        """The 'add' admin view for the Page model."""
+        """The ``add`` admin view for the :class:`Page <pages.models.Page>`."""
         extra_context = {
             'language': get_language_from_request(request),
             'page_languages': settings.PAGE_LANGUAGES,
@@ -332,7 +266,7 @@ class PageAdmin(admin.ModelAdmin):
                                                             extra_context)
 
     def has_add_permission(self, request):
-        """Return true if the current user has permission to add a new
+        """Return ``True`` if the current user has permission to add a new
         page."""
         if not settings.PAGE_PERMISSION:
             return super(PageAdmin, self).has_add_permission(request)
@@ -340,15 +274,15 @@ class PageAdmin(admin.ModelAdmin):
             return has_page_add_permission(request)
 
     def has_change_permission(self, request, obj=None):
-        """Return true if the current user has permission on the page.
-        Return the string 'All' if the user has all rights."""
+        """Return ``True`` if the current user has permission on the page.
+        Return the string ``All`` if the user has all rights."""
         if settings.PAGE_PERMISSION and obj is not None:
             return obj.has_page_permission(request)
         return super(PageAdmin, self).has_change_permission(request, obj)
 
     def has_delete_permission(self, request, obj=None):
-        """Return true if the current user has permission on the page.
-        Return the string 'All' if the user has all rights.
+        """Return ``True``  if the current user has permission on the page.
+        Return the string ``All`` if the user has all rights.
         """
         if settings.PAGE_PERMISSION and obj is not None:
             return obj.has_page_permission(request)
