@@ -3,11 +3,16 @@ from pages import settings
 from django.contrib import admin
 from django.forms import ModelForm
 from django.core.urlresolvers import get_mod_func
+from django.utils.translation import ugettext_lazy as _
+from django.core.urlresolvers import reverse
+from django.core.cache import cache
+from pages.models import Page, Content
+from pages.utils import get_placeholders
 
 def get_connected():
     if not settings.PAGE_CONNECTED_MODELS:
         return []
-    
+
     models = []
     for capp in settings.PAGE_CONNECTED_MODELS:
         model = {}
@@ -25,7 +30,7 @@ def get_connected():
             model['options'].update({'form': f})
 
         models.append((m, options))
-    
+
     return models
 
 def make_inline_admin(model_class, options):
