@@ -1,21 +1,37 @@
 # -*- coding: utf-8 -*-
-"""Django page CMS managers."""
+"""Django page CMS ``managers``."""
 import itertools, re
 from datetime import datetime
 from django.db import models, connection
-from django.contrib.sites.managers import CurrentSiteManager
 from django.contrib.sites.models import Site
 from django.db.models import Q
 from django.core.cache import cache
 
 from pages import settings
 from pages.utils import normalize_url, filter_link
+from django.contrib.auth.models import User
 
 class PageManager(models.Manager):
     """
     Page manager provide several filters to obtain pages :class:`QuerySet`
     that respect the page settings.
     """
+
+    def populate_pages(self, parent=None, child=5, depth=5):
+        """Create a population of pages for testing purpose."""
+        from pages.models import Content
+        author = User.objects.all()[0]
+        if depth==0:
+            return
+        p = self.model(parent=parent, author=author,
+            status=self.model.PUBLISHED)
+        p.save()
+        Content(body='page-'+str(p.id), type='title',
+            language='en-us', page=p).save()
+        Content(body='page-'+str(p.id), type='slug',
+            language='en-us', page=p).save()
+        for child in range(1, child):
+            self.populate_pages(parent=p, child=child, depth=(depth-1))
     
     def on_site(self, site_id=None):
         """Return a :class:`QuerySet` of pages that are published on the site
@@ -79,10 +95,11 @@ class PageManager(models.Manager):
             publication_end_date__lte=datetime.now())
 
     def from_path(self, complete_path, lang, exclude_drafts=True):
-        """Get a page according to the page's path."""
+        """Return a :class:`Page <pages.models.Page>` according to
+        the page's path."""
         from pages.models import Content, Page
         from pages.http import get_slug_and_relative_path
-        slug, path, lang = get_slug_and_relative_path(complete_path)
+        slug, path, lang = get_slug_and_relative_path(complete_path, lang)
         page_ids = Content.objects.get_page_ids_by_slug(slug)
         pages_list = self.on_site().filter(id__in=page_ids)
         if exclude_drafts:
@@ -93,15 +110,16 @@ class PageManager(models.Manager):
         # more than one page matching the slug, let's use the full url
         if len(pages_list) > 1:
             for page in pages_list:
-                if page.get_url(lang) == complete_path:
+                if page.get_complete_slug(lang) == complete_path:
                     return page
         return None
 
 class ContentManager(models.Manager):
-    """Content manager methods"""
+    """:class:`Content <pages.models.Content>` manager methods"""
 
     def sanitize(self, content):
-        """Sanitize a string in order to avoid possible XSS."""
+        """Sanitize a string in order to avoid possible XSS using
+        ``html5lib``."""
         import html5lib
         from html5lib import sanitizer
         p = html5lib.HTMLParser(tokenizer=sanitizer.HTMLSanitizer)
@@ -118,7 +136,7 @@ class ContentManager(models.Manager):
         :param body: the content of the Content object.
         """
         if settings.PAGE_SANITIZE_USER_INPUT:
-            body = self.sanitize(body)       
+            body = self.sanitize(body)
         try:
             content = self.filter(page=page, language=language,
                                   type=ctype).latest('creation_date')
@@ -187,8 +205,9 @@ class ContentManager(models.Manager):
 
         if language_fallback:
             for lang in settings.PAGE_LANGUAGES:
-                if lang[0] in content_dict:
-                    return filter_link(content_dict[lang[0]], page, lang[0], ctype)
+                if lang[0] in content_dict and content_dict[lang[0]]:
+                    return filter_link(content_dict[lang[0]], page, lang[0],
+                        ctype)
         return ''
 
     def get_content_slug_by_slug(self, slug):
@@ -247,11 +266,13 @@ class PagePermissionManager(models.Manager):
         return id_list
 
 class PageAliasManager(models.Manager):
-    """PageAlias manager."""
+    """:class:`PageAlias <pages.models.PageAlias>` manager."""
+    
     def from_path(self, request, path, lang):
         """
-        Resolve a request to an alias. returns a :class:`PageAlias <pages.models.PageAlias>`
-        if the url matches no page at all. The aliasing system supports plain
+        Resolve a request to an alias. returns a
+        :class:`PageAlias <pages.models.PageAlias>` if the url matches
+        no page at all. The aliasing system supports plain
         aliases (``/foo/bar``) as well as aliases containing GET parameters
         (like ``index.php?page=foo``).
 
