@@ -1,19 +1,19 @@
 # -*- coding: utf-8 -*-
-"""Django page CMS models."""
+"""Django page CMS ``models``."""
+import mptt
+
 from datetime import datetime
 from django.db import models
 from django.contrib.auth.models import User
-from django.template.defaultfilters import slugify
 from django.utils.translation import ugettext_lazy as _
 from django.utils.safestring import mark_safe
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
 from django.contrib.sites.models import Site
-import mptt
+
 from pages.utils import get_placeholders, normalize_url
 from pages.managers import PageManager, ContentManager
 from pages.managers import PagePermissionManager, PageAliasManager
-from pages.lib.BeautifulSoup import BeautifulSoup
 from pages import settings
 
 class Page(models.Model):
@@ -65,11 +65,11 @@ class Page(models.Model):
     
     parent = models.ForeignKey('self', null=True, blank=True, 
             related_name='children', verbose_name=_('parent'))
-    creation_date = models.DateTimeField(_('creation date'), editable=False, 
+    creation_date = models.DateTimeField(_('creation date'), editable=False,
             default=datetime.now)
     publication_date = models.DateTimeField(_('publication date'), 
-            null=True, blank=True, help_text=_('''When the page should go live.
-            Status must be "Published" for page to go live.'''))
+            null=True, blank=True, help_text=_('''When the page should go
+            live. Status must be "Published" for page to go live.'''))
     publication_end_date = models.DateTimeField(_('publication end date'), 
             null=True, blank=True, help_text=_('''When to expire the page.
             Leave empty to never expire.'''))
@@ -79,6 +79,9 @@ class Page(models.Model):
     status = models.IntegerField(_('status'), choices=STATUSES, default=DRAFT)
     template = models.CharField(_('template'), max_length=100, null=True,
             blank=True)
+
+    delegate_to = models.CharField(_('template'), max_length=100, null=True,
+            blank=True)
     
     # Disable could make site tests fail
     sites = models.ManyToManyField(Site, default=[settings.SITE_ID], 
@@ -97,13 +100,13 @@ class Page(models.Model):
         tags = fields.TagField(null=True)
 
     class Meta:
-        """Make sure the ordering is correct."""
+        """Make sure the default page ordering is correct."""
         ordering = ['tree_id', 'lft']
         verbose_name = _('page')
         verbose_name_plural = _('pages')
 
     def save(self, *args, **kwargs):
-        """Override the save method."""
+        """Override the default ``save`` method."""
         if not self.status:
             self.status = self.DRAFT
         # Published pages should always have a publication date
@@ -112,7 +115,8 @@ class Page(models.Model):
         # Drafts should not, unless they have been set to the future
         if self.status == self.DRAFT:
             if settings.PAGE_SHOW_START_DATE:
-                if self.publication_date and self.publication_date <= datetime.now():
+                if (self.publication_date and
+                        self.publication_date <= datetime.now()):
                     self.publication_date = None
             else:
                 self.publication_date = None
@@ -162,7 +166,7 @@ class Page(models.Model):
 
     def get_languages(self):
         """
-        Return a list of all existing languages for this page.
+        Return a list of all used languages for this page.
         """
         languages = cache.get(self.PAGE_LANGUAGES_KEY % (self.id))
         if languages:
@@ -177,13 +181,13 @@ class Page(models.Model):
         return languages
 
     def is_first_root(self):
-        """Return true if the page is the first root page."""
+        """Return ``True`` if the page is the first root page."""
         if self.parent:
             return False
         return Page.objects.root()[0].id == self.id
 
-    def get_absolute_url(self, language=None):
-        """Return the absolute page url. Add the language prefix if
+    def get_url_path(self, language=None):
+        """Return the URL's path component. Add the language prefix if
         ``PAGE_USE_LANGUAGE_PREFIX`` setting is set to ``True``.
 
         :param language: the wanted url language.
@@ -191,10 +195,23 @@ class Page(models.Model):
         url = reverse('pages-root')
         if settings.PAGE_USE_LANGUAGE_PREFIX:
             url += str(language) + '/'
-        return url + self.get_url(language)
+        return url + self.get_complete_slug(language)
 
-    def get_url(self, language=None):
-        """Return url of this page, adding all parent's slug."""
+    def get_absolute_url(self, language=None):
+        """Alias for `get_url_path`.
+
+        This method is only there for backward compatibility and will be
+        removed in a near futur.
+
+        :param language: the wanted url language.
+        """
+        return self.get_url_path(self, language=language)
+
+    def get_complete_slug(self, language=None):
+        """Return the complete slug of this page by concatenating
+        all parent's slugs.
+
+        :param language: the wanted slug language."""
         url = cache.get(self.PAGE_URL_KEY % (self.id, language))
         if url:
             return url
@@ -209,6 +226,16 @@ class Page(models.Model):
         
         return url
 
+    def get_url(self, language=None):
+        """Alias for `get_complete_slug`.
+
+        This method is only there for backward compatibility and will be
+        removed in a near futur.
+
+        :param language: the wanted url language.
+        """
+        return self.get_complete_slug(self, language=language)
+
     def slug(self, language=None, fallback=True):
         """
         Return the slug of the page depending on the given language.
@@ -264,7 +291,8 @@ class Page(models.Model):
         :data:`pages.settings.DEFAULT_PAGE_TEMPLATE` otherwise.
         """
         template = self.get_template()
-        for  t in settings.PAGE_TEMPLATES:
+        page_templates = settings.get_page_templates()
+        for t in page_templates:
             if t[0] == template:
                 return t[1]
         return template
@@ -285,11 +313,15 @@ class Page(models.Model):
             return False
 
     def has_broken_link(self):
+        """
+        Return ``True`` if the page have broken links to other pages
+        into the content.
+        """
         return cache.get(self.PAGE_BROKEN_LINK_KEY % self.id)
 
     def valid_targets(self, perms="All"):
-        """Return a :class:`QuerySet` of valid targets for moving a page into the
-        tree.
+        """Return a :class:`QuerySet` of valid targets for moving a page
+        into the tree.
 
         :param perms: the level of permission of the concerned user.
         """
@@ -330,7 +362,7 @@ except mptt.AlreadyRegistered:
 if settings.PAGE_PERMISSION:
     class PagePermission(models.Model):
         """
-        Page permission object
+        :class:`Page <pages.models.Page>` permission object
         """
         TYPES = (
             (0, _('All')),
@@ -352,8 +384,10 @@ if settings.PAGE_PERMISSION:
             return "%s :: %s" % (self.user,
                     unicode(PagePermission.TYPES[self.type][1]))
 
+
 class Content(models.Model):
-    """A block of content, tied to a page, for a particular language"""
+    """A block of content, tied to a :class:`Page <pages.models.Page>`,
+    for a particular language"""
     
     # languages could have five characters : Brazilian Portuguese is pt-br
     language = models.CharField(_('language'), max_length=5, blank=False)
@@ -373,9 +407,11 @@ class Content(models.Model):
     def __unicode__(self):
         return "%s :: %s" % (self.page.slug(), self.body[0:15])
 
+
 class PageAlias(models.Model):
-    """URL alias for a page"""
-    page = models.ForeignKey(Page, null=True, blank=True, verbose_name=_('page'))
+    """URL alias for a :class:`Page <pages.models.Page>`"""
+    page = models.ForeignKey(Page, null=True, blank=True,
+        verbose_name=_('page'))
     url = models.CharField(max_length=255, unique=True)
     objects = PageAliasManager()
     class Meta:
@@ -387,4 +423,5 @@ class PageAlias(models.Model):
         super(PageAlias, self).save(*args, **kwargs)
     
     def __unicode__(self):
-        return "%s => %s" % (self.url, self.page.get_url())
+        return "%s => %s" % (self.url, self.page.get_complete_slug())
+
