@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-"""Convenience module that provides access to custom settings for the
-``pages`` 
-application.  Provides default settings for the ``pages`` application when the
-project ``settings`` module does not contain the appropriate settings."""
+# Convenience module that provides access to custom settings for the
+# ``pages`` application.  Provides default settings for the ``pages``
+# application when the project ``settings`` module does not contain
+# the appropriate settings.
 from os.path import join
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
@@ -21,10 +21,27 @@ if DEFAULT_PAGE_TEMPLATE is None:
 #    ('pages/nice.html', 'nice one'),
 #    ('pages/cool.html', 'cool one'),
 # )
+#
+# One can also assign a callable (which should return the tuple) to this
+# variable to achieve dynamic template list e.g.:
+#
+# def _get_templates():
+#    ...
+#
+# PAGE_TEMPLATES = _get_templates
+
 PAGE_TEMPLATES = getattr(settings, 'PAGE_TEMPLATES', None)
-if PAGE_TEMPLATES is None:
+if (PAGE_TEMPLATES is None and 
+    not (isinstance(PAGE_TEMPLATES, str) or
+         isinstance(PAGE_TEMPLATES, unicode))):
     PAGE_TEMPLATES = ()
 
+def get_page_templates():
+    if callable(PAGE_TEMPLATES):
+        return PAGE_TEMPLATES()
+    else:
+        return PAGE_TEMPLATES
+
 # Set ``PAGE_PERMISSION`` to ``False`` if you do not wish to enable 
 # advanced hierarchic permissions on your pages.
 PAGE_PERMISSION = getattr(settings, 'PAGE_PERMISSION', True)
@@ -52,7 +69,7 @@ PAGE_UNIQUE_SLUG_REQUIRED = getattr(settings, 'PAGE_UNIQUE_SLUG_REQUIRED',
                                     False)
 
 # Set ``PAGE_CONTENT_REVISION`` to ``False`` to disable the recording of 
-# revision information about made to pages in the database
+# pages revision information in the database
 PAGE_CONTENT_REVISION = getattr(settings, 'PAGE_CONTENT_REVISION', True)
 
 # A list tuples that defines the languages that pages can be translated into.
@@ -67,18 +84,18 @@ PAGE_CONTENT_REVISION = getattr(settings, 'PAGE_CONTENT_REVISION', True)
 PAGE_LANGUAGES = getattr(settings, 'PAGE_LANGUAGES', settings.LANGUAGES)
 
 # Defines which language should be used by default.  If 
-# ``PAGE_DEFAULT_LANGUAGE`` not specified, then project 
+# ``PAGE_DEFAULT_LANGUAGE`` not specified, then project's
 # ``settings.LANGUAGE_CODE`` is used 
 PAGE_DEFAULT_LANGUAGE = getattr(settings, 'PAGE_DEFAULT_LANGUAGE', 
                                 settings.LANGUAGE_CODE)
 
 # PAGE_LANGUAGE_MAPPING should be assigned a function that takes a single
 # argument, the language code of the incoming browser request.  This function
-# maps the incoming client language code to another language code, presumably 
-# one for which you have translation strings.  This is most useful if your 
+# maps the incoming client language code to another language code, presumably
+# one for which you have translation strings.  This is most useful if your
 # project only has one set of translation strings for a language like Chinese,
-# which has several variants like ``zh-cn``, ``zh-tw``, ``zh-hk`, etc., but 
-# you want to provide your Chinese translations to all Chinese browsers, not 
+# which has several variants like ``zh-cn``, ``zh-tw``, ``zh-hk`, etc., but
+# you want to provide your Chinese translations to all Chinese browsers, not
 # just those with the exact ``zh-cn``
 # locale.
 #
@@ -109,16 +126,16 @@ PAGE_USE_LANGUAGE_PREFIX = getattr(settings, 'PAGE_USE_LANGUAGE_PREFIX',
 # Assign a list of placeholders to PAGE_CONTENT_REVISION_EXCLUDE_LIST
 # to exclude them from the revision process.
 PAGE_CONTENT_REVISION_EXCLUDE_LIST = getattr(settings,
-                                             'PAGE_CONTENT_REVISION_EXCLUDE_LIST', 
-                                             ())
+    'PAGE_CONTENT_REVISION_EXCLUDE_LIST', ()
+)
 
 # Set ``PAGE_SANITIZE_USER_INPUT`` to ``True`` to sanitize the user input with
 # ``html5lib``
 PAGE_SANITIZE_USER_INPUT = getattr(settings, 'PAGE_SANITIZE_USER_INPUT', False)
 
-# URL that handles pages' media and uses <MEDIA_ROOT>/pages by default.
-PAGES_MEDIA_URL = getattr(settings, 'PAGES_MEDIA_URL', join(settings.MEDIA_URL, 
-                                                            'pages/'))
+# URL that handles pages media and uses <MEDIA_ROOT>/pages by default.
+_media_url = getattr(settings, "STATIC_URL", settings.MEDIA_URL)
+PAGES_MEDIA_URL = getattr(settings, 'PAGES_MEDIA_URL', join(_media_url, 'pages/'))
 
 # Hide the slug's of the first root page ie: ``/home/`` becomes ``/``
 PAGE_HIDE_ROOT_SLUG = getattr(settings, 'PAGE_HIDE_ROOT_SLUG', False)
@@ -148,17 +165,15 @@ PAGE_SHOW_END_DATE = getattr(settings, 'PAGE_SHOW_END_DATE', False)
 #
 PAGE_CONNECTED_MODELS = getattr(settings, 'PAGE_CONNECTED_MODELS', False)
 
-# pagelink url validator agent
-PAGE_URL_VALIDATOR_USER_AGENT = getattr(settings, 'URL_VALIDATOR_USER_AGENT', False)
+# The page link filter enable a output filter on you content links. The goal
+# is to transform special page class into real links at the last moment.
+# This ensure that even if you have moved a page, the URL will remain correct.
+PAGE_LINK_FILTER = getattr(settings, 'PAGE_LINK_FILTER', False)
 
-# The page link filter enable a output filter on you content links. The goal is
-# to transform special page class into real links at the last moment. This ensure
-# that even if you have moved a page, the URL will be correct.
-PAGE_LINK_FILTER = getattr(settings, 'PAGE_LINK_FILTER', True)
+# This setting is a function that can be defined if you need to pass extra
+# context data to the pages templates.
+PAGE_EXTRA_CONTEXT = getattr(settings, 'PAGE_EXTRA_CONTEXT', None)
 
-# pagelink url validator user agent
-PAGE_URL_VALIDATOR_USER_AGENT = getattr(settings, 'URL_VALIDATOR_USER_AGENT', False)
-
-# This setting is a function that can be defined if you need to pass extra context
-# data to the pages templates.
-PAGE_EXTRA_CONTEXT = getattr(settings, 'PAGE_EXTRA_CONTEXT', None)
\ No newline at end of file
+# This setting is the name of a sub-folder where uploaded content, like
+# placeholder images, is placed.
+PAGE_UPLOAD_ROOT = getattr(settings, 'PAGE_UPLOAD_ROOT', 'upload')
