@@ -4,28 +4,40 @@ from django import template
 from django.utils.safestring import SafeUnicode, mark_safe
 from django.utils.translation import ugettext_lazy as _
 from django.template import Template, TemplateSyntaxError
-from django.conf import settings as global_settings
+#from django.forms import Widget, Textarea, ImageField, CharField
 import urllib
 
 from pages import settings
 from pages.models import Content, Page
-from pages.views import details
+from pages.placeholders import PlaceholderNode, ImagePlaceholderNode
+from pages.placeholders import VideoPlaceholderNode
+from pages.placeholders import parse_placeholder
 
 register = template.Library()
 
 PLACEHOLDER_ERROR = _("[Placeholder %(name)s had syntax error: %(error)s]")
 
+def get_page_from_string_or_id(page_string, lang):
+    """Return a Page object from a slug or an id."""
+    if type(page_string) == int:
+        return Page.objects.get(pk=int(page_string))
+    # if we have a string coming from some templates templates
+    if (isinstance(page_string, SafeUnicode) or
+        isinstance(page_string, unicode)):
+        if page_string.isdigit():
+            return Page.objects.get(pk=int(page_string))
+        return Page.objects.from_path(page_string, lang)
+    return page_string
+
 def _get_content(context, page, content_type, lang, fallback=True):
-    """Helper function used by placeholder nodes."""
+    """Helper function used by ``PlaceholderNode``."""
     if not page:
         return ''
 
     if not lang and 'lang' in context:
         lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
-    
-    # if the page is a SafeUnicode, try to use it like a slug
-    if isinstance(page, SafeUnicode) or isinstance(page, unicode):
-        page = Page.objects.from_path(page, lang)
+
+    page = get_page_from_string_or_id(page, lang)
 
     if not page:
         return ''
@@ -64,9 +76,11 @@ def pages_menu(context, page, url='/'):
     :param url: not used anymore.
     """
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    page = get_page_from_string_or_id(page, lang)
     path = context.get('path', None)
     site_id = None
-    children = page.get_children_for_frontend()
+    if page:
+        children = page.get_children_for_frontend()
     if 'current_page' in context:
         current_page = context['current_page']
     return locals()
@@ -82,9 +96,11 @@ def pages_sub_menu(context, page, url='/'):
     :param url: not used anymore.
     """
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    page = get_page_from_string_or_id(page, lang)
     path = context.get('path', None)
-    root = page.get_root()
-    children = root.get_children_for_frontend()
+    if page:
+        root = page.get_root()
+        children = root.get_children_for_frontend()
     if 'current_page' in context:
         current_page = context['current_page']
     return locals()
@@ -107,6 +123,7 @@ def pages_admin_menu(context, page, url='', level=None):
     has_permission = page.has_page_permission(request)
     PAGES_MEDIA_URL = settings.PAGES_MEDIA_URL
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    LANGUAGE_BIDI = context.get('LANGUAGE_BIDI', False)
 
     return locals()
 pages_admin_menu = register.inclusion_tag('admin/pages/page/menu.html',
@@ -124,10 +141,14 @@ def show_content(context, page, content_type, lang=None, fallback=True):
     
         {% show_content "my-page-slug" "title" %}
 
-    :param page: the page object
+    Or even the id of a page::
+
+        {% show_content 10 "title" %}
+
+    :param page: the page object, slug or id
     :param content_type: content_type used by a placeholder
     :param lang: the wanted language (default None, use the request object to know)
-    :param fallback: use fallback content
+    :param fallback: use fallback content from other language
     """
     return {'content':_get_content(context, page, content_type, lang,
                                                                 fallback)}
@@ -136,11 +157,10 @@ show_content = register.inclusion_tag('pages/content.html',
 
 def show_slug_with_level(context, page, lang=None, fallback=True):
     """Display slug with level by language."""
-    if not page:
-        return ''
     if not lang:
         lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
 
+    page = get_page_from_string_or_id(page, lang)
     if not page:
         return ''
 
@@ -161,16 +181,15 @@ def show_absolute_url(context, page, lang=None):
         {% show_absolute_url "my-page-slug" %}
 
     Keyword arguments:
-    :param page: the page object or a slug string
+    :param page: the page object, slug or id
     :param lang: the wanted language (defaults to `settings.PAGE_DEFAULT_LANGUAGE`)
     """
-    lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
-    # if the page is a SafeUnicode, try to use it like a slug
-    if isinstance(page, SafeUnicode) or isinstance(page, unicode):
-        page = Page.objects.from_path(page, lang)
+    if not lang:
+        lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    page = get_page_from_string_or_id(page, lang)
     if not page:
         return {'content':''}
-    url = page.get_absolute_url(language=lang)
+    url = page.get_url_path(language=lang)
     if url:
         return {'content':url}
     return {'content':''}
@@ -201,10 +220,11 @@ def pages_dynamic_tree_menu(context, page, url='/'):
     :param page: the current page
     :param url: not used anymore
     """
+    lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    page = get_page_from_string_or_id(page, lang)
     request = context['request']
     site_id = None
     children = None
-    lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
     if 'current_page' in context:
         current_page = context['current_page']
         # if this node is expanded, we also have to render its children
@@ -228,9 +248,11 @@ def pages_breadcrumb(context, page, url='/'):
     :param url: not used anymore
     """
     lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
+    page = get_page_from_string_or_id(page, lang)
     request = context['request']
     site_id = None
-    pages = page.get_ancestors()
+    if page:
+        pages_navigation = page.get_ancestors()
     return locals()
 pages_breadcrumb = register.inclusion_tag(
     'pages/breadcrumb.html',
@@ -248,14 +270,12 @@ class GetContentNode(template.Node):
         self.varname = varname
         self.lang = lang
     def render(self, context):
-        if self.lang is None:
-            lang = None
-        else:
-            lang = self.lang.resolve(context)
-        context[self.varname] = _get_content(context,
+        context[self.varname] = _get_content(
+            context,
             self.page.resolve(context),
             self.content_type.resolve(context),
-            lang)
+            self.lang
+        )
         return ''
 
 def do_get_content(parser, token):
@@ -273,7 +293,7 @@ def do_get_content(parser, token):
     
         {% get_content page type [lang] as name %}
 
-    :param page: the page object
+    :param page: the page object, slug or id
     :param type: content_type used by a placeholder
     :param name: name of the context variable to store the content in
     :param lang: the wanted language
@@ -297,9 +317,9 @@ do_get_content = register.tag('get_content', do_get_content)
 class LoadPagesNode(template.Node):
     """Load page node."""
     def render(self, context):
-        if 'pages' not in context:
+        if 'pages_navigation' not in context:
             pages = Page.objects.navigation().order_by("tree_id")
-            context.update({'pages': pages})
+            context.update({'pages_navigation': pages})
         if 'current_page' not in context:
             context.update({'current_page':None})
         return ''
@@ -312,7 +332,7 @@ def do_load_pages(parser, token):
     
         <ul>
             {% load_pages %}
-            {% for page in pages %}
+            {% for page in pages_navigation %}
                 {% pages_menu page %}
             {% endfor %}
         </ul>
@@ -321,115 +341,14 @@ def do_load_pages(parser, token):
 do_load_pages = register.tag('load_pages', do_load_pages)
 
 
-class PlaceholderNode(template.Node):
-    """This template node is used to output page content and
-    also in the admin to dynamically generate input fields.
-
-    :param name: the name of the placeholder you want to show/create
-    :param page: the optional page object
-    :param widget: the widget you want to use in the admin interface. Take
-        a look into :mod:`pages.admin.widgets` to see which widgets
-        are available.
-    :param parsed: if the ``parsed`` word is given, the content of the
-        placeholder is evaluated as template code, within the current context.
-    :param as_varname: if ``as_varname`` is defined, no value will be returned.
-        A variable will be created in the context with the defined name.
-    """
-    def handle_token(cls, parser, token):
-        """
-        Method that parse the template tag syntax following this syntax::
-
-            {% placeholder <name> [on <page>] [with <widget>] \
-[parsed] [as <varname>] %}
-        """
-        bits = token.split_contents()
-        count = len(bits)
-        error_string = '%r tag requires at least one argument' % bits[0]
-        if count <= 1:
-            raise template.TemplateSyntaxError(error_string)
-        name = bits[1]
-        remaining = bits[2:]
-        params = {}
-        while remaining:
-            bit = remaining[0]
-            if bit not in ('as', 'on', 'with', 'parsed'):
-                raise template.TemplateSyntaxError(
-                    "%r is not an correct option for a placeholder" % bit)
-            if bit in ('as', 'on', 'with'):
-                if len(remaining) < 2:
-                    raise template.TemplateSyntaxError(
-                    "Placeholder option '%s' need a parameter" % bit)
-                if bit == 'as':
-                    params['as_varname'] = remaining[1]
-                if bit == 'with':
-                    params['widget'] = remaining[1]
-                if bit == 'on':
-                    params['page'] = remaining[1]
-                remaining = remaining[2:]
-            else:
-                params['parsed'] = True
-                remaining = remaining[1:]
-        return cls(name, **params)
-    handle_token = classmethod(handle_token)
-    
-    def __init__(self, name, page=None, widget=None, parsed=False, as_varname=None):
-        self.page = page or 'current_page'
-        self.name = name
-        self.widget = widget
-        self.parsed = parsed
-        self.as_varname = as_varname
-        self.found_in_block = None
-
-    def render(self, context):
-        if not self.page in context:
-            return ''
-        # current_page can be set to None
-        if not context[self.page]:
-            return ''
-
-        lang = context.get('lang', settings.PAGE_DEFAULT_LANGUAGE)
-        content = Content.objects.get_content(context[self.page], lang,
-                                              self.name, True)
-        if not content:
-            return ''
-        if self.parsed:
-            try:
-                t = template.Template(content, name=self.name)
-                content = mark_safe(t.render(context))
-            except template.TemplateSyntaxError, error:
-                if global_settings.DEBUG:
-                    error = PLACEHOLDER_ERROR % {
-                        'name': self.name,
-                        'error': error,
-                    }
-                    if self.as_varname is None:
-                        return error
-                    context[self.as_varname] = error
-                    return ''
-                else:
-                    return ''
-        if self.as_varname is None:
-            return content
-        context[self.as_varname] = content
-        return ''
-
-    def __repr__(self):
-        return "<Placeholder Node: %s>" % self.name
-
 def do_placeholder(parser, token):
     """
+    Method that parse the placeholder template tag.
+    
     Syntax::
 
-        {% placeholder [name] %}
-        {% placeholder [name] parsed %}
-
-        {% placeholder [name] on [page]  %}
-        {% placeholder [name] with [widget] %}
-        {% placeholder [name] on [page] with [widget] %}
-
-        {% placeholder [name] on [page] parsed %}
-        {% placeholder [name] with [widget] parsed %}
-        {% placeholder [name] on [page] with [widget] parsed %}
+        {% placeholder <name> [on <page>] [with <widget>] \
+[parsed] [as <varname>] %}
 
     Example usage::
 
@@ -438,8 +357,24 @@ def do_placeholder(parser, token):
         {% placeholder welcome with TextArea parsed as welcome_text %}
         {% placeholder teaser on next_page with TextArea parsed %}
     """
-    return PlaceholderNode.handle_token(parser, token)
-
+    name, params = parse_placeholder(parser, token)
+    return PlaceholderNode(name, **params)
 register.tag('placeholder', do_placeholder)
 
 
+def do_imageplaceholder(parser, token):
+    """
+    Method that parse the imageplaceholder template tag.
+    """
+    name, params = parse_placeholder(parser, token)
+    return ImagePlaceholderNode(name, **params)
+register.tag('imageplaceholder', do_imageplaceholder)
+
+def do_videoplaceholder(parser, token):
+    """
+    Method that parse the imageplaceholder template tag.
+    """
+    name, params = parse_placeholder(parser, token)
+    return VideoPlaceholderNode(name, **params)
+register.tag('videoplaceholder', do_videoplaceholder)
+
