@@ -1,30 +1,19 @@
 # -*- coding: utf-8 -*-
 """Django page CMS test suite module"""
 import django
-from django.test import TestCase
-from pages import settings
-from pages.models import Page, Content, PagePermission, PageAlias
+from django.conf import settings
 from django.test.client import Client
 from django.template import Template, RequestContext, TemplateDoesNotExist
 from django.http import HttpResponse, HttpResponseRedirect
+from django.shortcuts import render_to_response
+
+from pages.models import Page, Content, PageAlias
+from pages.tests.testcase import TestCase
 
 class PagesTestCase(TestCase):
     """Django page CMS test suite class"""
-    fixtures = ['tests.json']
-    counter = 1
-
-    def get_new_page_data(self):
-        """Helper method for creating page datas"""
-        page_data = {'title':'test page %d' % self.counter, 
-            'slug':'test-page-%d' % self.counter, 'language':'en-us',
-            'sites':[2], 'status':Page.PUBLISHED,
-            # used to disable an error with connected models
-            'document_set-TOTAL_FORMS':0, 'document_set-INITIAL_FORMS':0,
-            }
-        self.counter = self.counter + 1
-        return page_data
-
-    def test_01_add_page(self):
+    
+    def test_add_page(self):
         """Test that the add admin page could be displayed via the
         admin"""
         c = Client()
@@ -33,7 +22,7 @@ class PagesTestCase(TestCase):
         self.assertEqual(response.status_code, 200)
 
 
-    def test_02_create_page(self):
+    def test_create_page(self):
         """Test that a page can be created via the admin."""
         #setattr(settings, "SITE_ID", 2)
         c = Client()
@@ -41,14 +30,16 @@ class PagesTestCase(TestCase):
         page_data = self.get_new_page_data()
         response = c.post('/admin/pages/page/add/', page_data)
         self.assertRedirects(response, '/admin/pages/page/')
-        slug_content = Content.objects.get_content_slug_by_slug(page_data['slug'])
+        slug_content = Content.objects.get_content_slug_by_slug(
+            page_data['slug']
+        )
         assert(slug_content is not None)
         page = slug_content.page
         self.assertEqual(page.title(), page_data['title'])
         self.assertEqual(page.slug(), page_data['slug'])
         self.assertNotEqual(page.last_modification_date, None)
 
-    def test_03_slug_collision(self):
+    def test_slug_collision(self):
         """Test a slug collision."""
         setattr(settings, "PAGE_UNIQUE_SLUG_REQUIRED", True)
 
@@ -57,7 +48,7 @@ class PagesTestCase(TestCase):
         page_data = self.get_new_page_data()
         response = c.post('/admin/pages/page/add/', page_data)
         self.assertRedirects(response, '/admin/pages/page/')
-        setattr(settings, "PAGE_UNIQUE_SLUG_REQUIRED", False)        
+        setattr(settings, "PAGE_UNIQUE_SLUG_REQUIRED", False)
         response = c.post('/admin/pages/page/add/', page_data)
         self.assertEqual(response.status_code, 200)
 
@@ -69,7 +60,7 @@ class PagesTestCase(TestCase):
         page2 = Content.objects.get_content_slug_by_slug(page_data['slug']).page
         self.assertNotEqual(page1.id, page2.id)
 
-    def test_04_details_view(self):
+    def test_details_view(self):
         """Test the details view"""
 
         c = Client()
@@ -99,7 +90,7 @@ class PagesTestCase(TestCase):
         response = c.get('/pages/test-page-2/')
         self.assertEqual(response.status_code, 200)
 
-    def test_05_edit_page(self):
+    def test_edit_page(self):
         """Test that a page can edited via the admin"""
         c = Client()
         c.login(username= 'batiste', password='b')
@@ -118,14 +109,14 @@ class PagesTestCase(TestCase):
         body = Content.objects.get_content(page, 'en-us', 'body')
         self.assertEqual(body, 'changed body')
 
-    def test_06_site_framework(self):
+    def test_site_framework(self):
         """Test the site framework, and test if it's possible to
         disable it"""
 
-        
-        # this kind of hacks doesn't work anymore
-        setattr(settings, "SITE_ID", 2)
-        setattr(settings, "PAGE_USE_SITE_ID", True)
+        # this is necessary to make the test pass
+        from pages import settings as pages_settings
+        setattr(pages_settings, "SITE_ID", 2)
+        setattr(pages_settings, "PAGE_USE_SITE_ID", True)
 
         c = Client()
         c.login(username= 'batiste', password='b')
@@ -151,7 +142,7 @@ class PagesTestCase(TestCase):
         content = Content.objects.get_content_slug_by_slug(page_data['slug'])
         self.assertEqual(content, None)
 
-        setattr(settings, "SITE_ID", 3)
+        setattr(pages_settings, "SITE_ID", 3)
         page = Content.objects.get_content_slug_by_slug(page_data['slug']).page
         self.assertEqual(page.sites.count(), 1)
         self.assertEqual(page.sites.all()[0].id, 3)
@@ -162,7 +153,7 @@ class PagesTestCase(TestCase):
 
         # without param
         self.assertEqual(Page.objects.on_site().count(), 1)
-        setattr(settings, "SITE_ID", 2)
+        setattr(pages_settings, "SITE_ID", 2)
         self.assertEqual(Page.objects.on_site().count(), 1)
 
         page_data = self.get_new_page_data()
@@ -174,12 +165,12 @@ class PagesTestCase(TestCase):
         self.assertEqual(Page.objects.on_site(2).count(), 2)
         self.assertEqual(Page.objects.on_site().count(), 2)
 
-        setattr(settings, "PAGE_USE_SITE_ID", False)
+        setattr(pages_settings, "PAGE_USE_SITE_ID", False)
 
         # we should get everything
         self.assertEqual(Page.objects.on_site().count(), 3)
 
-    def test_07_languages(self):
+    def test_languages(self):
         """Test post a page with different languages
         and test that the admin views works correctly."""
         c = Client()
@@ -199,6 +190,7 @@ class PagesTestCase(TestCase):
         self.assertRedirects(response, '/admin/pages/page/')
 
         page = Page.objects.all()[0]
+        self.assertEqual(page.get_languages(), ['en-us'])
 
         # this test only works in version superior of 1.0.2
         django_version =  django.get_version().rsplit()[0].split('.')
@@ -242,9 +234,8 @@ class PagesTestCase(TestCase):
         response = c.get('/pages/')
         self.assertContains(response, 'french title')
         self.assertContains(response, 'lang="fr-ch"')
-
         
-    def test_08_revision(self):
+    def test_revision(self):
         """Test that a page can edited several times."""
         c = Client()
         c.login(username= 'batiste', password='b')
@@ -267,7 +258,7 @@ class PagesTestCase(TestCase):
         
         self.assertEqual(Content.objects.get_content(page, 'en-us', 'body'), 'changed body 2')
 
-    def test_09_placeholder(self):
+    def test_placeholder(self):
         """
         Test that the placeholder is correctly displayed in
         the admin
@@ -284,7 +275,7 @@ class PagesTestCase(TestCase):
 
         self.assertContains(response, 'name="right-column"', 1)
 
-    def test_10_directory_slug(self):
+    def test_directory_slug(self):
         """
         Test diretory slugs
         """
@@ -324,7 +315,7 @@ class PagesTestCase(TestCase):
         response = c.get('/pages/same-slug/same-slug')
         self.assertContains(response, "children title", 2)
 
-    def test_11_show_content_tag(self):
+    def test_show_content_tag(self):
         """
         Test the {% show_content %} template tag
         """
@@ -345,7 +336,7 @@ class PagesTestCase(TestCase):
                             '{% show_content page "title" %}')
         self.assertEqual(template.render(context), page_data['title'])
 
-    def test_12_get_content_tag(self):
+    def test_get_content_tag(self):
         """
         Test the {% get_content %} template tag
         """
@@ -368,14 +359,14 @@ class PagesTestCase(TestCase):
         self.assertEqual(template.render(context), page_data['title'])
 
 
-    def test_17_request_mockup(self):
+    def test_request_mockup(self):
         from pages.utils import get_request_mock
         request = get_request_mock()
         self.assertEqual(hasattr(request, 'session'), True)
 
-    def test_18_tree_admin_interface(self):
+    def test_tree_admin_interface(self):
         """
-        Test that moving page in the tree is working properly
+        Test that moving/creating page in the tree is working properly
         using the admin interface
         """
         c = Client()
@@ -386,12 +377,14 @@ class PagesTestCase(TestCase):
         response = c.post('/admin/pages/page/add/', page_data)
         
         root_page = Content.objects.get_content_slug_by_slug('root').page
+        self.assertTrue(root_page.is_first_root())
         page_data['position'] = 'first-child'
         page_data['target'] = root_page.id
         page_data['slug'] = 'child-1'
         response = c.post('/admin/pages/page/add/', page_data)
         
         child_1 = Content.objects.get_content_slug_by_slug('child-1').page
+        self.assertFalse(child_1.is_first_root())
         
         page_data['slug'] = 'child-2'
         response = c.post('/admin/pages/page/add/', page_data)
@@ -400,20 +393,59 @@ class PagesTestCase(TestCase):
 
         self.assertEqual(str(Page.objects.all()),
             "[<Page: root>, <Page: child-2>, <Page: child-1>]")
+        # move page 1 in the first position
         response = c.post('/admin/pages/page/%d/move-page/' % child_1.id,
             {'position':'first-child', 'target':root_page.id})
 
         self.assertEqual(str(Page.objects.all()),
             "[<Page: root>, <Page: child-1>, <Page: child-2>]")
-        
+
+        # move page 2 in the first position
         response = c.post('/admin/pages/page/%d/move-page/' % child_2.id,
             {'position': 'left', 'target': child_1.id})
         
         self.assertEqual(str(Page.objects.all()),
             "[<Page: root>, <Page: child-2>, <Page: child-1>]")
 
+        # try to create a sibling with the same slug, via left, right
+        from pages import settings as pages_settings
+        setattr(pages_settings, "PAGE_UNIQUE_SLUG_REQUIRED", False)
+        page_data['target'] = child_2.id
+        page_data['position'] = 'left'
+        response = c.post('/admin/pages/page/add/', page_data)
+        self.assertEqual(response.status_code, 200)
 
-    def test_19_tree(self):
+        # try to create a sibling with the same slug, via first-child
+        page_data['target'] = root_page.id
+        page_data['position'] = 'first-child'
+        response = c.post('/admin/pages/page/add/', page_data)
+        self.assertEqual(response.status_code, 200)
+        # try to create a second page 2 in root
+        del page_data['target']
+        del page_data['position']
+
+        setattr(pages_settings, "PAGE_UNIQUE_SLUG_REQUIRED", True)
+        # cannot create because slug exists
+        response = c.post('/admin/pages/page/add/', page_data)
+        self.assertEqual(response.status_code, 200)
+        # Now it should work beause the page is not a sibling
+        setattr(pages_settings, "PAGE_UNIQUE_SLUG_REQUIRED", False)
+        response = c.post('/admin/pages/page/add/', page_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(Page.objects.count(), 4)
+        # Should not work because we already have sibling at the same level
+        response = c.post('/admin/pages/page/add/', page_data)
+        self.assertEqual(response.status_code, 200)
+
+        # try to change the page 2 slug into page 1
+        page_data['slug'] = 'child-1'
+        response = c.post('/admin/pages/page/%d/' % child_2.id, page_data)
+        self.assertEqual(response.status_code, 200)
+        setattr(pages_settings, "PAGE_UNIQUE_SLUG_REQUIRED", True)
+        response = c.post('/admin/pages/page/%d/' % child_2.id, page_data)
+        self.assertEqual(response.status_code, 200)
+
+    def test_tree(self):
         """
         Test that the navigation tree works properly with mptt
         """
@@ -462,10 +494,11 @@ class PagesTestCase(TestCase):
             "[<Page: page3>, <Page: page1>]")
             
     
-    def test_20_ajax_language(self):
+    def test_ajax_language(self):
+        """Test that language is working properly"""
         c = Client()
         c.login(username= 'batiste', password='b')
-        # Activate a language other thant settings.LANGUAGE_CODE
+        # Activate a language other than settings.LANGUAGE_CODE
         response = c.post('/i18n/setlang/', {'language':'fr-ch' })
         self.assertEqual(c.session.get('django_language', False), 'fr-ch')
         
@@ -474,10 +507,9 @@ class PagesTestCase(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertTrue('Auteur' in response.content)
         
-        # Create some pages (taken from test_18_tree_admin_interface)
+        # Create some pages (taken from test_tree_admin_interface)
         page_data = self.get_new_page_data()
         page_data['slug'] = 'root'
-
         response = c.post('/admin/pages/page/add/', page_data)
         
         root_page = Content.objects.get_content_slug_by_slug('root').page
@@ -519,7 +551,7 @@ class PagesTestCase(TestCase):
         # Make sure the content response we got was in french
         self.assertTrue('Auteur' in response.content)
 
-    def test_21_view_context(self):
+    def test_view_context(self):
         """
         Test that the default view can only return the context
         """
@@ -538,48 +570,7 @@ class PagesTestCase(TestCase):
         context = details(request, only_context=True)
         self.assertEqual(context['current_page'], page1)
 
-    def test_22_calculated_status_bug(self):
-        """
-        Test the issue 100
-        http://code.google.com/p/django-page-cms/issues/detail?id=100
-        """
-        setattr(settings, "PAGE_SHOW_START_DATE", True)
-        c = Client()
-        c.login(username= 'batiste', password='b')
-        page_data = self.get_new_page_data()
-        page_data['slug'] = 'page1'
-        # create a page for the example otherwise you will get a Http404 error
-        response = c.post('/admin/pages/page/add/', page_data)
-        page1 = Content.objects.get_content_slug_by_slug('page1').page
-
-        page1.status = Page.DRAFT
-        page1.save()
-
-        page1.get_calculated_status()
-        setattr(settings, "PAGE_SHOW_START_DATE", False)
-
-    def test_23_slug_bug(self):
-        """
-        Test the issue 97
-        http://code.google.com/p/django-page-cms/issues/detail?id=97
-        """
-        c = Client()
-        c.login(username= 'batiste', password='b')
-        page_data = self.get_new_page_data()
-        page_data['slug'] = 'page1'
-        # create a page for the example otherwise you will get a Http404 error
-        response = c.post('/admin/pages/page/add/', page_data)
-
-        response = c.get('/pages/page1/')
-        self.assertEqual(response.status_code, 200)
-
-        try:
-            response = c.get('/pages/toto/page1/')
-        except TemplateDoesNotExist, e:
-            if e.args != ('404.html',):
-                raise
-
-    def test_24_page_valid_targets(self):
+    def test_page_valid_targets(self):
         """Test page valid_targets method"""
         c = Client()
         c.login(username= 'batiste', password='b')
@@ -599,7 +590,7 @@ class PagesTestCase(TestCase):
         self.assertEqual(str(c1.valid_targets()),
                                             "[<Page: root>]")
 
-    def test_25_page_admin_view(self):
+    def test_page_admin_view(self):
         """Test page admin view"""
         c = Client()
         c.login(username= 'batiste', password='b')
@@ -608,10 +599,10 @@ class PagesTestCase(TestCase):
         response = c.post('/admin/pages/page/add/', page_data)
         page = Content.objects.get_content_slug_by_slug('page-1').page
         self.assertEqual(page.status, 1)
-        response = c.post('/admin/pages/page/%d/change-status-draft/' %
-            page.id)
+        response = c.post('/admin/pages/page/%d/change-status/' %
+            page.id, {'status':Page.DRAFT})
         page = Content.objects.get_content_slug_by_slug('page-1').page
-        self.assertEqual(page.status, 0)
+        self.assertEqual(page.status, Page.DRAFT)
 
         url = '/admin/pages/page/%d/modify-content/title/en-us/' % page.id
         response = c.post(url, {'content': 'test content'})
@@ -630,7 +621,7 @@ class PagesTestCase(TestCase):
         response = c.get(url)
         self.assertEqual(response.status_code, 200)
         
-    def test_26_page_alias(self):
+    def test_page_alias(self):
         """Test page aliasing system"""
 
         c = Client()
@@ -671,12 +662,37 @@ class PagesTestCase(TestCase):
         # calling via its alias must cause redirect
         response = c.get('/pages/index.php?page=downloads')
         self.assertRedirects(response, '/pages/downloads-page', 301)
+       
+    def test_page_redirect_to(self):
+        """Test page redirected to an other page."""
+
+        client = Client()
+        client.login(username= 'batiste', password='b')
+
+        # create some pages
+        page1 = self.create_new_page(client)
+        page2 = self.create_new_page(client)
+        
+        page1.redirect_to = page2
+        page1.save()
+
+        # now check whether you go to the target page.
+        response = client.get(page1.get_url_path())
+        self.assertRedirects(response, page2.get_url_path(), 301)
+
+    def test_page_redirect_to_url(self):
+        """Test page redirected to external url."""
+
+        client = Client()
+        client.login(username= 'batiste', password='b')
+        page1 = self.create_new_page(client)
+        url = 'http://code.google.com/p/django-page-cms/'
+        page1.redirect_to_url = url
+        page1.save()
+
+        # now check whether we can retrieve the page.
+        response = client.get(page1.get_url_path())
+        self.assertTrue(response.status_code == 301)
+        self.assertTrue(response['Location'] == url)
 
-    def test_27_bug_152(self):
-        # http://code.google.com/p/django-page-cms/issues/detail?id=152
-        from pages.utils import get_placeholders
-        self.assertEqual(
-            str(get_placeholders('tests/test1.html')),
-            "[<Placeholder Node: body>]"
-        )
         
\ No newline at end of file
